<?php

namespace Drupal\imager_widget\Plugin\Field\FieldWidget;

use Drupal\Core\Form\FormStateInterface;
use Drupal\image\Plugin\Field\FieldWidget\ImageWidget;
use Drupal\Core\Url;
use Drupal\file\Entity\File;
use Drupal\Core\File\FileSystemInterface;

/**
 * Plugin implementation of the 'imager_widget' widget.
 *
 * @FieldWidget(
 *   id = "imager_widget",
 *   label = @Translation("Imager Widget"),
 *   field_types = {
 *     "image"
 *   }
 * )
 */
class ImagerWidget extends ImageWidget {

  /**
   * The config factory.
   *
   * Subclasses should use the self::config() method, which may be overridden to
   * address specific needs when loading config, rather than this property
   * directly. See \Drupal\Core\Form\ConfigFormBase::config() for an example of
   * this.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Form API callback: Processes a crop_image field element.
   *
   * Expands the image_image type to include the alt and title fields.
   *
   * This method is assigned as a #process callback in formElement() method.
   *
   * @return array
   *   The elements with parents fields.
   */
  public static function process($element, FormStateInterface $form_state, $form) {
    if ($element['#files']) {
      foreach ($element['#files'] as $file) {
        $element['imager_widget_wrapper'] = [
          '#type' => 'fieldset',
          '#title' => t('Edit your image'),
        ];
        $element['imager_widget_wrapper']['imager_widget'] = [
          '#markup' => '<a class="add-new-imager button" href="#">' . t('Start Editing') . '</a><div class="description">' . t('To rotate or crop image: IMPORTANT: Click Save icon at top right corner of the image to save your edited photo. Click Save button at the bottom of profile to save the profile before navigating away from the page.') . '</div>
            <div class="imagers"></div>',
        ];
        $element['imager_widget_wrapper']['imager_output'] = [
          '#type' => 'textarea',
          '#prefix' => '<div class="hidden">',
          '#suffix' => '</div>',
        ];
      }
      $element['#attached']['library'][] = 'imager_widget/imagerjs';
      $element['#attached']['drupalSettings']['path']['baseUrl'] = Url::fromRoute('<front>', [], ['absolute' => TRUE])->toString();
    }

    return parent::process($element, $form_state, $form);
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {

   // Get original file
      
      $i = 0;
      foreach ($values as $file_item) {
        
        if(count($file_item['fids'])>0){
          
          $original_file = File::load($file_item['fids'][0]);
          $original_file_name = $original_file->getFilename(); // string (26) "alutech-am-5000kit-1-o.jpg"
          // Get data from saved image data.
          
          $file_content = $file_item['imager_widget_wrapper']['imager_output'];
          
          if(strlen($file_content)>0){
            list($type, $data) = explode(';', $file_content);
            list(, $data) = explode(',', $data);
            $data_decoded = base64_decode($data);
            list(, $mimetype) = explode(':', $type);
   
           // Save new file to drupal file managed.
            $file = file_save_data($data_decoded, 'public://' . $original_file_name, FileSystemInterface::EXISTS_RENAME);
          
            $file->setMimeType($mimetype);
            $file->save();
      // Set new fid to field values.
            if (is_object($file)) {
              $values[$i]['fids'][0] = $file->id();
              unset($values[$i]['imager_output']);
            }
          }
           
        }
       
    $i++;
   }
    return parent::massageFormValues($values, $form, $form_state);
  }

}
